@@ -18,7 +18,7 @@
  *
  * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
  *
- $Id: battstat_applet.c,v 1.34 2002/03/03 16:46:39 kevinv Exp $
+ $Id: battstat_applet.c,v 1.36 2002/04/28 16:57:43 kevinv Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -797,25 +797,50 @@ void
 about_cb (PanelApplet *widget, gpointer data)
 {
    GtkWidget   *about_box;
-   char        *authors[] = { "Jrgen Pehrson <jp@spektr.eu.org>", 
-			      "Lennart Poettering <lennart@poettering.de> (Linux ACPI support)",
-			      "Seth Nickell <snickell@stanford.edu> (GNOME2 port)",
-			      NULL };
+   GdkPixbuf   *pixbuf;
+   GError      *error = NULL;
+   gchar       *file;
+   
+   const gchar *authors[] = {
+	/* if your charset supports it, please replace the "o" in
+	 * "Jorgen" into U00F6 */
+	_("Jorgen Pehrson <jp@spektr.eu.org>"), 
+	"Lennart Poettering <lennart@poettering.de> (Linux ACPI support)",
+	"Seth Nickell <snickell@stanford.edu> (GNOME2 port)",
+	NULL
+   };
+
+   const gchar *documenters[] = {
+	NULL
+   };
+
+   const gchar *translator_credits = _("translator_credits");
+   
+   file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "battstat.png", FALSE, NULL);
+   pixbuf = gdk_pixbuf_new_from_file (file, &error);
+   g_free (file);
+   
+   if (error) {
+   	g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
+	g_error_free (error);
+   }
    
    about_box = gnome_about_new (
 				/* The long name of the applet in the About dialog.*/
-				_("Battery status utility"), 
+				_("Battery Charge Monitor"), 
 				VERSION,
 				_("(C) 2000 The Gnulix Society, (C) 2002 Free Software Foundation"),
 				_("This utility show the status of your laptop battery."),
-				(const char **) authors,
-				/* Longer description of the applet in the About dialog.*/
-				NULL,
-				NULL,
-				NULL);
-   /* FIXME
-				"battstat-tesla.xpm");
-   */
+				authors,
+				documenters,
+				strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+				pixbuf);
+   
+   if (pixbuf) 
+   	gdk_pixbuf_unref (pixbuf);
+   
+   gtk_window_set_wmclass (GTK_WINDOW (about_box), "battery charge monitor", "Batter Charge Monitor");
+   gnome_window_icon_set_from_file (GTK_WINDOW (about_box), GNOME_ICONDIR"/battstat.png");
    gtk_widget_show (about_box);
 }
 
