@@ -46,16 +46,6 @@
 #include "images/cdplayer-next.xpm"
 #include "images/cdplayer-eject.xpm"
 
-#if defined(sun) || defined(__sun__)
-#if defined(SVR4) || defined(__svr4__)
-#   define DEV_PATH "/vol/dev/aliases/cdrom0"
-#else
-#   define DEV_PATH "/dev/rcd0"
-#endif
-#else
-#   define DEV_PATH "/dev/cdrom"
-#endif
-
 #define TIMEOUT_VALUE 500
 
 /* Function prototypes */
@@ -94,10 +84,14 @@ static void cdplayer_prev(GtkWidget * w, gpointer data);
 static void cdplayer_next(GtkWidget * w, gpointer data);
 static void cdplayer_eject(GtkWidget * w, gpointer data);
 
+static void set_atk_relation(GtkWidget *label, GtkWidget *entry);
+static void set_atk_name_description(GtkWidget *widget, const gchar *name, const gchar *description);
+static void make_applet_accessible(CDPlayerData *cd);
+
 /* Bonobo Verbs for our popup menu */
 static const BonoboUIVerb applet_menu_verbs [] = {
     BONOBO_UI_UNSAFE_VERB ("RunGTCD", start_gtcd_cb),
-    BONOBO_UI_UNSAFE_VERB ("Properties", properties_cb),
+    BONOBO_UI_UNSAFE_VERB ("Preferences", properties_cb),
     BONOBO_UI_UNSAFE_VERB ("Help", help_cb),
     BONOBO_UI_UNSAFE_VERB ("About", about_cb),
     BONOBO_UI_VERB_END
@@ -105,7 +99,7 @@ static const BonoboUIVerb applet_menu_verbs [] = {
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_CDPlayerApplet_Factory",
 			     PANEL_TYPE_APPLET,
-                             "CD-Player-Applet",
+                             "cdplayer",
                              "0",
                               applet_factory,
                               NULL)
@@ -142,7 +136,10 @@ applet_fill (PanelApplet *applet)
     g_signal_connect_after (G_OBJECT (cdplayer), "realize", G_CALLBACK (cdplayer_realize), cd);
     gtk_widget_show(cdplayer);
     
-    tooltips = gtk_tooltips_new ();
+    tooltips = gtk_tooltips_new (); 
+    g_object_ref (tooltips);
+    gtk_object_sink (GTK_OBJECT (tooltips));
+    g_object_set_data (G_OBJECT (cd->panel.applet), "tooltips", tooltips);
 
     cd->panel.box = NULL;
     cd->panel.play_control.stop = control_button_factory(stop_xpm, G_CALLBACK(cdplayer_stop), cd);
@@ -155,10 +152,11 @@ applet_fill (PanelApplet *applet)
     gtk_tooltips_set_tip (tooltips, cd->panel.track_control.prev, _("Previous Track"), NULL);
     cd->panel.track_control.next = control_button_factory(next_xpm, G_CALLBACK(cdplayer_next), cd);
     gtk_tooltips_set_tip (tooltips, cd->panel.track_control.next, _("Next Track"), NULL);
-    led_init();
     led_create_widgets(&cd->panel.time, &cd->panel.track_control.display, (gpointer)cd);
 
     gtk_container_add (GTK_CONTAINER (applet), cdplayer);
+    gtk_tooltips_set_tip (tooltips, cd->panel.applet, _("CD Player"), NULL);
+    make_applet_accessible (cd); 
 
     /* panel_applet_add_preferences (applet, "/schemas/apps/cdplayer-applet/prefs", NULL); */
     cdplayer_load_config(cd);
@@ -182,6 +180,7 @@ applet_fill (PanelApplet *applet)
 
     g_signal_connect (component, "ui-event", G_CALLBACK (ui_component_event), cd);
     gtk_widget_show (GTK_WIDGET(applet));
+    setup_box(cd);
     return(TRUE);
 }
 
@@ -189,7 +188,7 @@ static void
 cdplayer_load_config(CDPlayerData *cd)
 {
     g_free(cd->devpath);
-    cd->devpath = panel_applet_gconf_get_string(PANEL_APPLET(cd->panel.applet), "device-path", NULL);
+    cd->devpath = panel_applet_gconf_get_string(PANEL_APPLET(cd->panel.applet), "device_path", NULL);
     if (!cd->devpath || !strcmp(cd->devpath, "none"))
     {
         g_free(cd->devpath);
@@ -200,23 +199,33 @@ cdplayer_load_config(CDPlayerData *cd)
 static void
 cdplayer_save_config(CDPlayerData *cd)
 {
-    panel_applet_gconf_set_string(PANEL_APPLET(cd->panel.applet), "device-path", cd->devpath, NULL);
+    panel_applet_gconf_set_string(PANEL_APPLET(cd->panel.applet), "device_path", cd->devpath, NULL);
 }
 
 static void
 cdplayer_destroy(GtkWidget * widget, gpointer data)
 {
     CDPlayerData *cd = data;
+    GtkTooltips *tooltips;
 
     if (cd->timeout != 0)
         gtk_timeout_remove(cd->timeout);
     cd->timeout = 0;
     cd_close (cd);
 
+    tooltips = g_object_get_data (G_OBJECT (cd->panel.applet), "tooltips");
+    if (tooltips) {
+        g_object_unref (tooltips);
+        g_object_set_data (G_OBJECT (cd->panel.applet), "tooltips", NULL);
+    }
+    
+    if (cd->time_description)
+        g_free(cd->time_description); 
+    if (cd->track_description)
+        g_free(cd->track_description);
     g_free (cd->devpath);
     cd->devpath = NULL;
     g_free(cd);
-    led_done();
 }
 
 static void
@@ -243,7 +252,12 @@ cdplayer_timeout_callback(gpointer data)
 static void
 start_gtcd_cb(GtkWidget *w, gpointer data)
 {
-    g_spawn_command_line_async ("gnome-cd", NULL);
+    GError *err = NULL;
+    g_spawn_command_line_async ("gnome-cd", &err);
+    if (err) {
+      g_print ("%s\n", err->message);
+      g_error_free (err);
+    }
 }
 
 static void
@@ -294,12 +308,32 @@ focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
     return FALSE;
 }
 
+static void
+set_default_device (GtkButton *button, gpointer data)
+{
+    CDPlayerData *cd = data;
+    GtkWidget *entry = g_object_get_data (G_OBJECT (button), "entry");
+    
+    if (!strcmp(cd->devpath, DEV_PATH))
+    	return;
+    	
+    cd_close(cd);
+    if (cd->devpath)
+        g_free(cd->devpath);
+    cd->devpath = g_strdup(DEV_PATH);
+    cd_try_open(cd);
+    gtk_entry_set_text (GTK_ENTRY (entry), cd->devpath);
+    cdplayer_save_config(cd);  
+}
+    
+/* FIXME: change the fn name later to reflect Preferences */
 static void
 properties_cb (GtkWidget *w, gpointer data)
 {
     CDPlayerData *cd;
     GtkWidget *dialog;
     GtkWidget *box;
+    GtkWidget *button;
     GtkWidget *hbox;
     GtkWidget *label;
     GtkWidget *image;
@@ -308,33 +342,46 @@ properties_cb (GtkWidget *w, gpointer data)
 
     cd = (CDPlayerData *) data;
 
-    dialog = gtk_dialog_new_with_buttons(_("CD Player Properties"),
+    dialog = gtk_dialog_new_with_buttons(_("CD Player Preferences"),
                                          NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                          NULL);
+    gtk_dialog_set_default_response(GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
     box = GTK_DIALOG(dialog)->vbox;
-
+    
     hbox = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD);
     gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
 
-    image = gtk_image_new_from_stock(GTK_STOCK_CDROM, GTK_ICON_SIZE_DIALOG);
+    image = gtk_image_new_from_stock(GTK_STOCK_CDROM, GTK_ICON_SIZE_BUTTON);
     gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 10);
     gtk_widget_show(image);
+    set_atk_name_description(image, _("Disc Image"), _("An image of a cd-rom disc"));
 
-    label = gtk_label_new(_("Device Path:"));
+    label = gtk_label_new_with_mnemonic(_("Device _Path:"));
     gtk_misc_set_padding (GTK_MISC (label), GNOME_PAD_SMALL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
 
     entry = gtk_entry_new();
-    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 4);
     gtk_widget_show(entry);
     gtk_entry_set_text(GTK_ENTRY(entry), cd->devpath);
+    set_atk_name_description(entry, _("Device Path"), _("Set the device path here"));
+    set_atk_relation(label, entry);
+    
+    button = gtk_button_new_with_mnemonic (_("Use _Default"));
+    gtk_box_pack_start (GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_object_set_data (G_OBJECT (button), "entry", entry);
+    gtk_widget_show (button);
+    
     g_signal_connect (G_OBJECT (entry), "activate",
     		      G_CALLBACK (activate_cb), cd);
     g_signal_connect (G_OBJECT (entry), "focus_out_event",
     		      G_CALLBACK (focus_out_cb), cd);
+    g_signal_connect (G_OBJECT (button), "clicked",
+    		      G_CALLBACK (set_default_device), cd);
     
     g_signal_connect (G_OBJECT (dialog), "response",
     		      G_CALLBACK (response_cb), NULL);
@@ -346,13 +393,24 @@ properties_cb (GtkWidget *w, gpointer data)
 static void
 help_cb (GtkWidget *w, gpointer data)
 {
+    GError *error = NULL;
 
+    gnome_help_display ("cdplayer", NULL, &error);
+    if (error) {
+        g_warning ("help error: %s\n", error->message);
+        g_error_free (error);
+    }
+
+    return;
 }
 
 static void
 about_cb(GtkWidget *w, gpointer data)
 {
     static GtkWidget   *about     = NULL;
+    GdkPixbuf	       *pixbuf;
+    GError	       *error     = NULL;
+    gchar	       *file;
 
     static const gchar *authors[] =
     {
@@ -362,6 +420,13 @@ about_cb(GtkWidget *w, gpointer data)
         NULL
     };
 
+    const gchar *documenters[] =
+    {
+	    NULL
+    };
+
+    const gchar *translator_credits = _("translator_credits");
+
     if (about != NULL)
     {
         gdk_window_show(about->window);
@@ -369,12 +434,28 @@ about_cb(GtkWidget *w, gpointer data)
         return;
     }
     
-    about = gnome_about_new (_("CD Player Applet"), VERSION,
+    file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-cdplayer-icon.png", FALSE, NULL);
+    pixbuf = gdk_pixbuf_new_from_file (file, &error);
+    g_free (file);
+    
+    if (error) {
+    	g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
+	g_error_free (error);
+    }
+    
+    about = gnome_about_new (_("CD Player"), VERSION,
                              _("(C) 1997 The Free Software Foundation\n" \
                                "(C) 2001 Chris Phelps (GNOME 2 Port)"),
                              _("The CD Player applet is a simple audio CD player for your panel"),
-                             authors,  NULL, NULL, NULL);
+                             authors,
+			     documenters,
+			     strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			     pixbuf);
+    if (pixbuf)
+    	gdk_pixbuf_unref (pixbuf);
 
+    gtk_window_set_wmclass (GTK_WINDOW (about), "cd player", "CD Player");
+    gnome_window_icon_set_from_file (GTK_WINDOW (about), GNOME_ICONDIR"/gnome-cdplayer-icon.png");	
     g_signal_connect (G_OBJECT(about), "destroy",
                       G_CALLBACK(gtk_widget_destroyed), &about);
     gtk_widget_show (about);
@@ -512,7 +593,6 @@ control_button_factory(gchar * pixmap_data[], GCallback func, CDPlayerData * cd)
 
     w = gtk_button_new();
     GTK_WIDGET_UNSET_FLAGS(w, GTK_CAN_DEFAULT);
-    GTK_WIDGET_UNSET_FLAGS(w, GTK_CAN_FOCUS);
     pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **)pixmap_data);
     image = gtk_image_new_from_pixbuf(pixbuf);
     if (pixbuf)
@@ -600,6 +680,7 @@ static void
 cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd)
 {
     cdrom_device_status_t stat;
+    gboolean description = FALSE;
 
     if (cd_try_open(cd)) {
         if (cdrom_get_status(cd->cdrom_device, &stat) == DISC_NO_ERROR)
@@ -612,6 +693,7 @@ cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd)
                          stat.relative_address.second,
                          cd->panel.track_control.display,
                          stat.track);
+                    description = TRUE;
                     break;
                 case DISC_PAUSED:
                     led_paused(cd->panel.time,
@@ -619,6 +701,7 @@ cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd)
                            stat.relative_address.second,
                            cd->panel.track_control.display,
                            stat.track);
+                    description = TRUE;
                     break;
                 case DISC_COMPLETED:
                 case DISC_STOP:
@@ -635,8 +718,30 @@ cd_panel_update(GtkWidget * cdplayer, CDPlayerData * cd)
     {
         led_nodisc(cd->panel.time, cd->panel.track_control.display);
     }
+    if (description)
+    {
+        cd->time_description =
+            g_strdup_printf("Time elapsed is %d minutes and %d seconds",
+                stat.relative_address.minute,
+                stat.relative_address.second);
+        cd->track_description =
+            g_strdup_printf("Current track number is %d",
+                stat.track);
+        set_atk_name_description(cd->panel.time, _("Elapsed time"),
+            cd->time_description);
+        set_atk_name_description(cd->panel.track_control.display,
+            _("Track number"), cd->track_description);
+        g_free (cd->time_description);
+        g_free (cd->track_description);
+    }
+    else
+    {
+        set_atk_name_description(cd->panel.time, _("Elapsed time"), "");
+        set_atk_name_description(cd->panel.track_control.display,
+            _("Track number"), "");
+    }
     return;
-    cdplayer = NULL;
+    cdplayer = NULL; 
 }
 
 
@@ -735,3 +840,65 @@ cdplayer_eject(GtkWidget * w, gpointer data)
     return;
         w = NULL;
 }
+
+static void
+make_applet_accessible(CDPlayerData *cd)
+{
+    /* Check if gail is loaded */
+    if (GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(cd->panel.play_control.stop))
+        == FALSE) return;
+	
+    set_atk_name_description(cd->panel.play_control.stop, _("Stop"), 
+        _("Click this button to Stop playing the CD"));
+    set_atk_name_description(cd->panel.play_control.play_pause, _("Play / Pause"), 
+        _("Click this button to Play or Pause the CD"));
+    set_atk_name_description(cd->panel.play_control.eject, _("Eject"), 
+        _("Click this button to eject the CD"));
+    set_atk_name_description(cd->panel.track_control.prev, _("Previous Track"),
+        _("Click this button to Play the previous track in the CD"));
+    set_atk_name_description(cd->panel.track_control.next, _("Next Track"),
+        _("Click this button to Play the next track in the CD"));
+    set_atk_name_description(cd->panel.applet, _("CD Player"), 
+        _("The CD Player applet is a simple audio CD player for your panel"));
+}
+
+static void
+set_atk_name_description(GtkWidget *widget, const gchar *name,
+    const gchar *description)
+{	
+    AtkObject *aobj;
+	
+    aobj = gtk_widget_get_accessible(widget);
+    /* Check if gail is loaded */
+    if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
+        return; 
+    atk_object_set_name(aobj, name);
+    atk_object_set_description(aobj, description);
+}
+
+static void
+set_atk_relation(GtkWidget *label, GtkWidget *widget)
+{
+    AtkObject *atk_widget;
+    AtkObject *atk_label;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+    AtkObject *targets[1];
+
+    atk_widget = gtk_widget_get_accessible(widget);
+    atk_label = gtk_widget_get_accessible(label);
+
+    /* Set the label-for relation */
+    gtk_label_set_mnemonic_widget(GTK_LABEL (label), widget);	
+
+    /* Check if gail is loaded */
+    if (GTK_IS_ACCESSIBLE (atk_widget) == FALSE)
+        return;
+
+    /* Set the labelled-by relation */
+    relation_set = atk_object_ref_relation_set(atk_widget);
+    targets[0] = atk_label;
+    relation = atk_relation_new(targets, 1, ATK_RELATION_LABELLED_BY);
+    atk_relation_set_add(relation_set, relation);
+    g_object_unref(G_OBJECT (relation));
+}
