@@ -1,4 +1,4 @@
-/* $Id: gweather-pref.c,v 1.35 2002/04/09 12:55:57 baddog Exp $ */
+/* $Id: gweather-pref.c,v 1.38 2002/05/01 21:01:10 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -708,6 +708,7 @@ static void gweather_pref_create (GWeatherApplet *gw_applet)
 				      		   GTK_DIALOG_DESTROY_WITH_PARENT,
 				      		   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				      		   NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (gw_applet->pref), GTK_RESPONSE_CLOSE);
     gtk_widget_set_usize (gw_applet->pref, -2, 280);
     gtk_window_set_policy (GTK_WINDOW (gw_applet->pref), TRUE, TRUE, FALSE);
     
@@ -1025,12 +1026,8 @@ void gweather_pref_load (GWeatherApplet *gw_applet)
 
 static void help_cb (void)
 {
-/*
-    GnomeHelpMenuEntry help_entry = { "gweather_applet",
-				      "index.html#GWEATHER-PREFS" };
-
-    gnome_help_display (NULL, &help_entry);
-*/
+    GError *error = NULL;
+    gnome_help_display("gweather","gweather-prefs",&error);
 }
 
 void gweather_pref_run (GWeatherApplet *gw_applet)
