@@ -781,14 +781,8 @@ mixer_help_cb (BonoboUIComponent *uic,
 	       gpointer           data,
 	       const gchar       *verbname)
 {
-#if 0
-	GnomeHelpMenuEntry help_entry = {
-		"volume-control-applet",
-		"index.html"
-	};
-
-	gnome_help_display (NULL, &help_entry);
-#endif
+        GError *error = NULL;
+	gnome_help_display("mixer",NULL,&error);
 }
 
 /* Dummy callback to get rid of a warning, for now. */
@@ -912,7 +906,7 @@ mixer_applet_create (PanelApplet *applet)
 	/* Try to find some mixers - sdtaudiocontrol is on Solaris and is needed
 	** because gnome-volume-meter doesn't work */
 	if (run_mixer_cmd == NULL) 
-		run_mixer_cmd = g_find_program_in_path ("gnome-volume-meter");
+		run_mixer_cmd = g_find_program_in_path ("gnome-volume-control");
 	if (run_mixer_cmd == NULL) 
 		run_mixer_cmd = g_find_program_in_path ("gmix");
 	if (run_mixer_cmd == NULL) 
@@ -1039,7 +1033,7 @@ mixer_applet_factory (PanelApplet *applet,
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_MixerApplet_Factory",
 			     PANEL_TYPE_APPLET,
-			     "Volume control applet",
+			     "mixer",
 			     "0",
 			     mixer_applet_factory,
 			     NULL)
