@@ -15,6 +15,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libbonobo.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -33,6 +37,7 @@
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-monitor.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 
 #include "panel-menu.h"
 #include "panel-menu-common.h"
@@ -42,10 +47,10 @@
 
 static const gchar *directory_menu_xml =
 	"    <placeholder name=\"ChildItem\">\n"
-	"        <menuitem name=\"Action\" verb=\"Action\" label=\"%s Properties...\"\n"
+	"        <menuitem name=\"Action\" verb=\"Action\" label=\"%s Menu Preferences...\"\n"
 	"                  pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
 	"%s"
-	"        <menuitem name=\"Remove\" verb=\"Remove\" label=\"Remove %s\"\n"
+	"        <menuitem name=\"Remove\" verb=\"Remove\" label=\"Remove %s Menu\"\n"
 	"                  pixtype=\"stock\" pixname=\"gtk-close\"/>\n"
 	"        <separator/>"
 	"    </placeholder>";
@@ -82,6 +87,7 @@ static GtkWidget *panel_menu_directory_edit_dialog_new (gchar *title,
 							GtkWidget **nentry,
 							GtkWidget **pentry,
 							GtkWidget **spin);
+static gboolean panel_menu_directory_warn_about_path (char *path);
 
 static gint object_counter = 0;
 
@@ -194,6 +200,7 @@ void
 panel_menu_directory_set_path (PanelMenuEntry *entry, gchar *path)
 {
 	PanelMenuDirectory *directory;
+	gchar *old_path = NULL;
 	gchar *full_path = NULL;
 	gchar *label;
 	gchar *show_label;
@@ -202,23 +209,30 @@ panel_menu_directory_set_path (PanelMenuEntry *entry, gchar *path)
 	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
 
 	directory = (PanelMenuDirectory *) entry->data;
-	if (directory->path)
-		g_free (directory->path);
-	if (directory->monitor)
+	/* Save it here, we don't want to free till the end BUG #77003 */
+	old_path = directory->path;
+	if (directory->monitor) {
 		gnome_vfs_monitor_cancel (directory->monitor);
+		directory->monitor = NULL;
+	}
 	directory->path = panel_menu_common_build_full_path (path, "");;
 	regenerate_menus_cb (NULL, entry, NULL);
-	gnome_vfs_monitor_add (&directory->monitor,
+	if (gnome_vfs_monitor_add (&directory->monitor,
 			       directory->path,
 			       GNOME_VFS_MONITOR_DIRECTORY,
 			       directory_changed_cb,
-			       entry);
-	if (directory->monitor)
+			       entry) == GNOME_VFS_OK)
 		g_print ("monitor successfully installed for %s\n",
 			  directory->path);
-	else
+	else {
+		directory->monitor = NULL;
 		g_print ("monitor installation failed for directory %s\n",
 			  directory->path);
+	}
+	if (old_path) {
+		g_free (directory->path);
+		directory->path = NULL;
+	}
 }
 
 static void
@@ -231,30 +245,6 @@ directory_changed_cb(GnomeVFSMonitorHandle *handle,
 	PanelMenuEntry *entry;
 	entry = (PanelMenuEntry *) user_data;
 
-	switch (event_type) {
-		case GNOME_VFS_MONITOR_EVENT_CHANGED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_CHANGED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_DELETED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_DELETED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
-			g_print ("GNOME_VFS_MONITOR_EVENT_STARTEXECUTING");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
-			g_print ("GNOME_VFS_MONITOR_EVENT_STOPEXECUTING");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_CREATED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_CREATED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED");
-			break;
-		default:
-			break;
-	}
-	g_print (" (%s)", info_uri);
-	g_print ("\n");
 	regenerate_menus_cb (NULL, entry, NULL);
 }
 
@@ -318,14 +308,15 @@ panel_menu_directory_destroy (PanelMenuEntry *entry)
 	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
 
 	directory = (PanelMenuDirectory *) entry->data;
+	if (directory->monitor)
+		gnome_vfs_monitor_cancel (directory->monitor);
 	if (directory->name)
 		g_free (directory->name);
 	if (directory->path)
 		g_free (directory->path);
-	if (directory->monitor)
-		gnome_vfs_monitor_cancel (directory->monitor);
 	gtk_widget_destroy (directory->directory);
 	g_free (directory);
+	g_print ("directory item completely removed\n");
 }
 
 GtkWidget *
@@ -333,8 +324,8 @@ panel_menu_directory_get_widget (PanelMenuEntry *entry)
 {
 	PanelMenuDirectory *directory;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY, NULL);
 
 	directory = (PanelMenuDirectory *) entry->data;
 	return directory->directory;
@@ -474,34 +465,29 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	}
 }
 
-void
-panel_menu_directory_new_with_dialog (PanelMenu *panel_menu)
+static void
+handle_create_dir_response (GtkDialog *dialog, gint response, PanelMenu *panel_menu)
 {
-	GtkWidget *dialog;
-	GtkWidget *name_entry;
-	GtkWidget *path_entry;
-	GtkWidget *level_spin;
-	gint response;
-	gchar *name;
-	gchar *path;
-	gint level;
-
-	PanelMenuEntry *entry;
-
-	dialog = panel_menu_directory_edit_dialog_new (_
-						       ("Create directory item..."),
-						       &name_entry, &path_entry,
-						       &level_spin);
-	gtk_entry_set_text (GTK_ENTRY (name_entry), _("Home"));
-	gtk_entry_set_text (GTK_ENTRY (path_entry), g_get_home_dir ());
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (level_spin), 1.0);
-	gtk_widget_show (dialog);
-	gtk_widget_grab_focus (path_entry);
-
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (response == GTK_RESPONSE_OK) {
+
+		GtkWidget *name_entry;
+		GtkWidget *path_entry;
+		GtkWidget *level_spin;
+		PanelMenuEntry *entry;
+		gchar *name;
+		gchar *path;
+		gint level;
+
+		name_entry = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "uri-name"));
 		name = (gchar *) gtk_entry_get_text (GTK_ENTRY (name_entry));
+
+		path_entry = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "uri-path"));
 		path = (gchar *) gtk_entry_get_text (GTK_ENTRY (path_entry));
+		
+		if (!panel_menu_directory_warn_about_path(path))
+			return;
+
+		level_spin = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "depth-level"));
 		level = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
 							  (level_spin));
 		entry = panel_menu_directory_new (panel_menu, name, path,
@@ -513,7 +499,82 @@ panel_menu_directory_new_with_dialog (PanelMenu *panel_menu)
 		panel_menu_config_save_layout (panel_menu);
 		panel_menu_directory_save_config (entry);
 	}
-	gtk_widget_destroy (dialog);
+
+	gtk_widget_destroy (GTK_WIDGET(dialog));
+}
+
+void
+panel_menu_directory_new_with_dialog (PanelMenu *panel_menu)
+{
+	GtkWidget *dialog;
+	GtkWidget *name_entry;
+	GtkWidget *path_entry;
+	GtkWidget *level_spin;
+
+	dialog = panel_menu_directory_edit_dialog_new (_
+						       ("Create directory item..."),
+						       &name_entry, &path_entry,
+						       &level_spin);
+
+        g_signal_connect (G_OBJECT (dialog), "response",
+                          G_CALLBACK (handle_create_dir_response),
+                          panel_menu);
+
+	gtk_entry_set_text (GTK_ENTRY (name_entry), _("Home"));
+	gtk_entry_set_text (GTK_ENTRY (path_entry), g_get_home_dir ());
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (level_spin), 1.0);
+	gtk_widget_show (dialog);
+	gtk_widget_grab_focus (path_entry);
+}
+
+static void
+handle_edit_dir_response (GtkDialog *dialog, gint response, PanelMenuEntry *entry)
+{
+	if (response == GTK_RESPONSE_OK) {
+
+		PanelMenuDirectory *directory;
+		GtkWidget *name_entry;
+		GtkWidget *path_entry;
+		GtkWidget *level_spin;
+
+		const gchar *new_name;
+		const gchar *new_path;
+		gint new_level;
+
+		gboolean changed = FALSE;
+
+        	directory = (PanelMenuDirectory *) entry->data;
+
+		name_entry = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "uri-name"));
+		new_name = gtk_entry_get_text (GTK_ENTRY (name_entry));
+
+		path_entry = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "uri-path"));
+		new_path = gtk_entry_get_text (GTK_ENTRY (path_entry));
+
+		level_spin = GTK_WIDGET(g_object_get_data (G_OBJECT (dialog), "depth-level"));
+		new_level =
+			gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
+							  (level_spin));
+		if (strcmp (directory->name, new_name)) {
+			panel_menu_directory_set_name (entry, (gchar *)new_name);
+			changed = TRUE;
+		}
+		if (strcmp (directory->path, new_path)) {
+			directory->level = new_level;
+			panel_menu_directory_warn_about_path((gchar *)new_path);
+			panel_menu_directory_set_path (entry, (gchar *)new_path);
+			changed = TRUE;
+		} else if (directory->level != new_level) {
+			directory->level = new_level;
+			panel_menu_directory_set_path (entry, directory->path);
+			changed = TRUE;
+		}
+		if (changed) {
+			panel_menu_directory_save_config (entry);
+		}
+	}
+
+	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
 static void
@@ -524,11 +585,6 @@ change_directory_cb (GtkWidget *widget, PanelMenuEntry *entry, const gchar *verb
 	GtkWidget *name_entry;
 	GtkWidget *path_entry;
 	GtkWidget *level_spin;
-	gint response;
-	gchar *new_name;
-	gchar *new_path;
-	gint new_level;
-	gboolean changed = FALSE;
 
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
@@ -538,6 +594,10 @@ change_directory_cb (GtkWidget *widget, PanelMenuEntry *entry, const gchar *verb
 						       ("Edit directory item..."),
 						       &name_entry, &path_entry,
 						       &level_spin);
+        g_signal_connect (G_OBJECT (dialog), "response",
+                          G_CALLBACK (handle_edit_dir_response),
+                          entry);
+
 	gtk_entry_set_text (GTK_ENTRY (name_entry), directory->name);
 	gtk_entry_set_text (GTK_ENTRY (path_entry), directory->path);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (level_spin),
@@ -545,33 +605,42 @@ change_directory_cb (GtkWidget *widget, PanelMenuEntry *entry, const gchar *verb
 
 	gtk_widget_show (dialog);
 	gtk_widget_grab_focus (name_entry);
+}
+
+
+static void
+handle_browse_response (GtkDialog *dialog, gint response, GtkFileSelection *fsel)
+{
+	GtkEntry *entry;
+	const gchar *new_path;
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (response == GTK_RESPONSE_OK) {
-		new_name =
-			(gchar *) gtk_entry_get_text (GTK_ENTRY (name_entry));
-		new_path =
-			(gchar *) gtk_entry_get_text (GTK_ENTRY (path_entry));
-		new_level =
-			gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
-							  (level_spin));
-		if (strcmp (directory->name, new_name)) {
-			panel_menu_directory_set_name (entry, new_name);
-			changed = TRUE;
+		new_path = gtk_file_selection_get_filename (fsel);
+		if (new_path) {
+			entry = GTK_ENTRY(g_object_get_data (G_OBJECT (dialog), "target"));
+			gtk_entry_set_text (entry, new_path);
 		}
-		if (strcmp (directory->path, new_path)) {
-			directory->level = new_level;
-			panel_menu_directory_set_path (entry, new_path);
-			changed = TRUE;
-		} else if (directory->level != new_level) {
-			directory->level = new_level;
-			panel_menu_directory_set_path (entry, directory->path);
-			changed = TRUE;
-		}
-		if (changed)
-			panel_menu_directory_save_config (entry);
 	}
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy (GTK_WIDGET(dialog));
+}
+
+static void
+browse_callback (GtkButton *button, GtkEntry *entry)
+{
+	GtkWidget *fsel;
+	const gchar *old_path;
+
+	fsel = gtk_file_selection_new ("Choose a directory...");
+	gtk_window_set_transient_for (GTK_WINDOW (fsel), GTK_WINDOW (
+				      gtk_widget_get_toplevel (GTK_WIDGET (entry))));
+	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
+	old_path = gtk_entry_get_text (entry);
+	gtk_file_selection_set_filename (GTK_FILE_SELECTION (fsel), old_path);
+	gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (fsel), FALSE);
+	g_object_set_data (G_OBJECT (fsel), "target", entry);
+	g_signal_connect (G_OBJECT (fsel), "response",
+			  G_CALLBACK (handle_browse_response), fsel);
+	gtk_widget_show (fsel);
 }
 
 static GtkWidget *
@@ -582,48 +651,67 @@ panel_menu_directory_edit_dialog_new (gchar *title, GtkWidget **nentry,
 	GtkWidget *box;
 	GtkWidget *hbox;
 	GtkWidget *label;
+	GtkWidget *browse;
 
 	dialog = gtk_dialog_new_with_buttons (title,
-					      NULL, GTK_DIALOG_MODAL,
+					      NULL, 0,
 					      GTK_STOCK_CLOSE,
 					      GTK_RESPONSE_CLOSE, GTK_STOCK_OK,
 					      GTK_RESPONSE_OK, NULL);
 
 	box = GTK_DIALOG (dialog)->vbox;
+	gtk_container_set_border_width (GTK_CONTAINER (box), 5);
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 5);
 	gtk_widget_show (hbox);
-	label = gtk_label_new (_("Name:"));
+	label = gtk_label_new_with_mnemonic (_("_Name:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show (label);
 	*nentry = gtk_entry_new_with_max_length (50);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), *nentry);
+	set_relation(*nentry, GTK_LABEL(label));
 	gtk_box_pack_start (GTK_BOX (hbox), *nentry, TRUE, TRUE, 5);
 	gtk_widget_show (*nentry);
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 5);
 	gtk_widget_show (hbox);
-	label = gtk_label_new (_("Path:"));
+	label = gtk_label_new_with_mnemonic (_("_Path:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show (label);
 	*pentry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), *pentry, TRUE, TRUE, 5);
 	gtk_widget_show (*pentry);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), *pentry);
+	set_relation(*pentry, GTK_LABEL(label));
+	browse = gtk_button_new_with_label (_("..."));
+	add_tooltip(browse, "Browse to select a directory");
+	gtk_box_pack_start (GTK_BOX (hbox), browse, FALSE, FALSE, 5);
+	g_signal_connect (G_OBJECT (browse), "clicked",
+			  G_CALLBACK(browse_callback), *pentry);
+	gtk_widget_show (browse);
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 5);
 	gtk_widget_show (hbox);
-	label = gtk_label_new (_("Depth level:"));
+	label = gtk_label_new_with_mnemonic (_("_Depth level:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show (label);
 	*spin = gtk_spin_button_new (GTK_ADJUSTMENT
 				     (gtk_adjustment_new (1, 1.0, 10, 1, 1, 1)),
 				     1, 0);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), *spin);
+	set_relation(*spin, GTK_LABEL(label));
 	gtk_box_pack_end (GTK_BOX (hbox), *spin, TRUE, TRUE, 5);
 	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (*spin),
 					   GTK_UPDATE_ALWAYS);
 	gtk_widget_show (*spin);
+
+        g_object_set_data (G_OBJECT (dialog), "uri-name", *nentry);
+        g_object_set_data (G_OBJECT (dialog), "uri-path", *pentry);
+        g_object_set_data (G_OBJECT (dialog), "depth-level", *spin);
+
 	return dialog;
 }
 
@@ -636,14 +724,15 @@ panel_menu_directory_save_config (PanelMenuEntry *entry)
 	gchar *id;
 	gchar *key;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY, NULL);
 
 	panel_menu = entry->parent;
 	applet = panel_menu->applet;
 	directory = (PanelMenuDirectory *) entry->data;
 
 	id = g_strdup_printf ("directory%d", directory->id);
+	g_print ("Saving config for %s as %s - %s - %d.\n", id, directory->name, directory->path, directory->level);
 	key = g_strdup_printf ("%s/name", id);
 	panel_applet_gconf_set_string (applet, key, directory->name, NULL);
 	g_free (key);
@@ -682,3 +771,38 @@ panel_menu_directory_remove_config (PanelMenuEntry *entry)
 	g_free (key);
 	g_object_unref (G_OBJECT (client));
 }
+
+static void
+message_dlg_response (GtkDialog *dialog, gint response, gpointer data)
+{
+	gtk_widget_destroy (GTK_WIDGET(dialog));
+}
+
+static gboolean
+panel_menu_directory_warn_about_path (char *path)
+{
+	struct stat sbuf;
+	GtkWidget *message_dlg;
+
+	stat(path, &sbuf);
+
+	if (S_ISDIR (sbuf.st_mode) == FALSE ){
+
+		message_dlg = gtk_message_dialog_new (NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_WARNING,
+				GTK_BUTTONS_OK,
+				_("Path specified is currently not a valid directory."));
+		gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
+		gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+
+        	g_signal_connect (G_OBJECT (message_dlg), "response",
+                          G_CALLBACK (message_dlg_response),
+                          NULL);
+		gtk_widget_show_all (message_dlg);
+		
+		return FALSE;
+	}
+	
+	return TRUE;
+}
