@@ -15,6 +15,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libbonobo.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -40,9 +44,9 @@
 
 static const gchar *documents_menu_xml =
 	"    <placeholder name=\"ChildItem\">\n"
-	"        <menuitem name=\"Action\" verb=\"Action\" label=\"Rename %s...\"\n"
+	"        <menuitem name=\"Action\" verb=\"Action\" label=\"Rename %s Menu...\"\n"
 	"                  pixtype=\"stock\" pixname=\"gtk-convert\"/>\n"
-	"        <menuitem name=\"Remove\" verb=\"Remove\" label=\"Remove %s\"\n"
+	"        <menuitem name=\"Remove\" verb=\"Remove\" label=\"Remove %s Menu\"\n"
 	"                  pixtype=\"stock\" pixname=\"gtk-close\"/>\n"
 	"        <separator/>" "    </placeholder>";
 
@@ -226,8 +230,8 @@ panel_menu_documents_get_widget (PanelMenuEntry *entry)
 {
 	PanelMenuDocuments *documents;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DOCUMENTS);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DOCUMENTS, NULL);
 
 	documents = (PanelMenuDocuments *) entry->data;
 	return (documents->documents);
@@ -391,7 +395,7 @@ panel_menu_documents_new_with_dialog (PanelMenu *panel_menu)
 
 	dialog = panel_menu_common_single_entry_dialog_new (_
 							    ("Create documents item..."),
-							    _("Name:"),
+							    _("_Name:"),
 							    _("Documents"),
 							    &name_entry);
 	gtk_widget_show (dialog);
@@ -427,7 +431,7 @@ rename_documents_cb (GtkWidget *widget, PanelMenuEntry *entry,
 	documents = (PanelMenuDocuments *) entry->data;
 
 	dialog = panel_menu_common_single_entry_dialog_new (_("Rename documents item..."),
-							    _("Name:"),
+							    _("_Name:"),
 							    documents->name,
 							    &name_entry);
 	gtk_widget_show (dialog);
@@ -453,8 +457,8 @@ panel_menu_documents_save_config (PanelMenuEntry *entry)
 	gchar *id;
 	gchar *key;
 
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DOCUMENTS);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DOCUMENTS, NULL);
 
 	panel_menu = entry->parent;
 	applet = panel_menu->applet;
